@@ -16,8 +16,8 @@ fn main() {
     let mut update = SystemStage::parallel();
 
     // Add systems to increase the counter and to print out the current value
-    update.add_system(increase_counter.label(CounterSystem::Increase));
-    update.add_system(print_counter.after(CounterSystem::Increase));
+    update.add_system(increase_counter);
+    update.add_system(print_counter.after(increase_counter));
     schedule.add_stage("update", update);
 
     for iteration in 1..=10 {
@@ -32,12 +32,6 @@ struct Counter {
     pub value: i32,
 }
 
-// System label to enforce a run order of our systems
-#[derive(SystemLabel, Debug, Clone, PartialEq, Eq, Hash)]
-enum CounterSystem {
-    Increase,
-}
-
 fn increase_counter(mut counter: ResMut<Counter>) {
     if rand::thread_rng().gen_bool(0.5) {
         counter.value += 1;
