@@ -4,7 +4,7 @@ use crate::{
     bundle::Bundle,
     component::Component,
     entity::{Entities, Entity},
-    world::World,
+    world::{FromWorld, World},
 };
 use bevy_utils::tracing::{error, warn};
 pub use command_queue::CommandQueue;
@@ -78,7 +78,7 @@ impl<'w, 's> Commands<'w, 's> {
     ///         // adds a single component to the entity
     ///         .insert(Label("hello world"));
     /// }
-    /// # example_system.system();
+    /// # bevy_ecs::system::assert_is_system(example_system);
     /// ```
     pub fn spawn<'a>(&'a mut self) -> EntityCommands<'w, 's, 'a> {
         let entity = self.entities.reserve_entity();
@@ -107,7 +107,7 @@ impl<'w, 's> Commands<'w, 's> {
     /// Spawns a [`Bundle`] without pre-allocating an [`Entity`]. The [`Entity`] will be allocated
     /// when this [`Command`] is applied.
     pub fn spawn_and_forget(&mut self, bundle: impl Bundle) {
-        self.queue.push(Spawn { bundle })
+        self.queue.push(Spawn { bundle });
     }
 
     /// Creates a new entity with the components contained in `bundle`.
@@ -156,7 +156,7 @@ impl<'w, 's> Commands<'w, 's> {
     ///         // or insert single components like this:
     ///         .insert(Label("hello world"));
     /// }
-    /// # example_system.system();
+    /// # bevy_ecs::system::assert_is_system(example_system);
     /// ```
     pub fn spawn_bundle<'a, T: Bundle>(&'a mut self, bundle: T) -> EntityCommands<'w, 's, 'a> {
         let mut e = self.spawn();
@@ -188,7 +188,7 @@ impl<'w, 's> Commands<'w, 's> {
     ///         // adds a single component to the entity
     ///         .insert(Label("hello world"));
     /// }
-    /// # example_system.system();
+    /// # bevy_ecs::system::assert_is_system(example_system);
     /// ```
     #[track_caller]
     pub fn entity<'a>(&'a mut self, entity: Entity) -> EntityCommands<'w, 's, 'a> {
@@ -232,7 +232,7 @@ impl<'w, 's> Commands<'w, 's> {
     ///     ),
     /// ]);
     /// # }
-    /// # system.system();
+    /// # bevy_ecs::system::assert_is_system(system);
     /// ```
     pub fn spawn_batch<I>(&mut self, bundles_iter: I)
     where
@@ -261,9 +261,45 @@ impl<'w, 's> Commands<'w, 's> {
         self.queue.push(InsertOrSpawnBatch { bundles_iter });
     }
 
+    /// Inserts a resource with standard starting values to the [`World`].
+    ///
+    /// If the resource already exists, nothing happens.
+    ///
+    /// The value given by the [`FromWorld::from_world`] method will be used.
+    /// Note that any resource with the `Default` trait automatically implements `FromWorld`,
+    /// and those default values will be here instead.
+    ///
+    /// See [`World::init_resource`] for more details.
+    /// Note that commands do not take effect immediately.
+    /// When possible, prefer the equivalent methods on `App` or `World`.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use bevy_ecs::prelude::*;
+    /// #
+    /// # #[derive(Default)]
+    /// # struct Scoreboard {
+    /// #     current_score: u32,
+    /// #     high_score: u32,
+    /// # }
+    /// #
+    /// # fn system(mut commands: Commands) {
+    /// commands.init_resource::<Scoreboard>();
+    /// # }
+    /// # system.system();
+    /// ```
+    pub fn init_resource<R: Resource + FromWorld>(&mut self) {
+        self.queue.push(InitResource::<R> {
+            _phantom: PhantomData::<R>::default(),
+        });
+    }
+
     /// Inserts a resource to the [`World`], overwriting any previous value of the same type.
     ///
     /// See [`World::insert_resource`] for more details.
+    /// Note that commands do not take effect immediately.
+    /// When possible, prefer the equivalent methods on `App` or `World`.
     ///
     /// # Example
     ///
@@ -281,10 +317,10 @@ impl<'w, 's> Commands<'w, 's> {
     ///     high_score: 0,
     /// });
     /// # }
-    /// # system.system();
+    /// # bevy_ecs::system::assert_is_system(system);
     /// ```
-    pub fn insert_resource<T: Resource>(&mut self, resource: T) {
-        self.queue.push(InsertResource { resource })
+    pub fn insert_resource<R: Resource>(&mut self, resource: R) {
+        self.queue.push(InsertResource { resource });
     }
 
     /// Removes a resource from the [`World`].
@@ -304,10 +340,10 @@ impl<'w, 's> Commands<'w, 's> {
     /// # fn system(mut commands: Commands) {
     /// commands.remove_resource::<Scoreboard>();
     /// # }
-    /// # system.system();
+    /// # bevy_ecs::system::assert_is_system(system);
     /// ```
-    pub fn remove_resource<T: Resource>(&mut self) {
-        self.queue.push(RemoveResource::<T> {
+    pub fn remove_resource<R: Resource>(&mut self) {
+        self.queue.push(RemoveResource::<R> {
             phantom: PhantomData,
         });
     }
@@ -345,7 +381,7 @@ impl<'w, 's> Commands<'w, 's> {
     ///         },
     ///     });
     /// }
-    /// # add_combat_stats_system.system();
+    /// # bevy_ecs::system::assert_is_system(add_combat_stats_system);
     /// ```
     pub fn add<C: Command>(&mut self, command: C) {
         self.queue.push(command);
@@ -369,9 +405,10 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
     /// fn my_system(mut commands: Commands) {
     ///     let entity_id = commands.spawn().id();
     /// }
-    /// # my_system.system();
+    /// # bevy_ecs::system::assert_is_system(my_system);
     /// ```
     #[inline]
+    #[must_use = "Omit the .id() call if you do not need to store the `Entity` identifier."]
     pub fn id(&self) -> Entity {
         self.entity
     }
@@ -405,7 +442,7 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
     ///         defense: Defense(20),
     ///     });
     /// }
-    /// # add_combat_stats_system.system();
+    /// # bevy_ecs::system::assert_is_system(add_combat_stats_system);
     /// ```
     pub fn insert_bundle(&mut self, bundle: impl Bundle) -> &mut Self {
         self.commands.add(InsertBundle {
@@ -417,16 +454,6 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
 
     /// Adds a single [`Component`] to the entity.
     ///
-    /// See [`EntityMut::insert`](crate::world::EntityMut::insert) for more
-    /// details.
-    ///
-    /// # Warning
-    ///
-    /// It's possible to call this with a bundle, but this is likely not intended and
-    /// [`Self::insert_bundle`] should be used instead. If `with` is called with a bundle, the
-    /// bundle itself will be added as a component instead of the bundles' inner components each
-    /// being added.
-    ///
     /// # Example
     ///
     /// `Self::insert` can be chained with [`Commands::spawn`].
@@ -449,7 +476,7 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
     ///     commands.spawn().insert_bundle((Component1, Component2));
     ///     commands.spawn_bundle((Component1, Component2));
     /// }
-    /// # example_system.system();
+    /// # bevy_ecs::system::assert_is_system(example_system);
     /// ```
     pub fn insert(&mut self, component: impl Component) -> &mut Self {
         self.commands.add(Insert {
@@ -479,7 +506,7 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
     /// fn remove_combat_stats_system(mut commands: Commands, player: Res<PlayerEntity>) {
     ///     commands.entity(player.entity).remove_bundle::<CombatBundle>();
     /// }
-    /// # remove_combat_stats_system.system();
+    /// # bevy_ecs::system::assert_is_system(remove_combat_stats_system);
     /// ```
     pub fn remove_bundle<T>(&mut self) -> &mut Self
     where
@@ -508,7 +535,7 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
     /// fn convert_enemy_system(mut commands: Commands, enemy: Res<TargetEnemy>) {
     ///     commands.entity(enemy.entity).remove::<Enemy>();
     /// }
-    /// # convert_enemy_system.system();
+    /// # bevy_ecs::system::assert_is_system(convert_enemy_system);
     /// ```
     pub fn remove<T>(&mut self) -> &mut Self
     where
@@ -539,12 +566,12 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
     /// {
     ///     commands.entity(character_to_remove.entity).despawn();
     /// }
-    /// # remove_character_system.system();
+    /// # bevy_ecs::system::assert_is_system(remove_character_system);
     /// ```
     pub fn despawn(&mut self) {
         self.commands.add(Despawn {
             entity: self.entity,
-        })
+        });
     }
 
     /// Returns the underlying [`Commands`].
@@ -553,6 +580,15 @@ impl<'w, 's, 'a> EntityCommands<'w, 's, 'a> {
     }
 }
 
+impl<F> Command for F
+where
+    F: FnOnce(&mut World) + Send + Sync + 'static,
+{
+    fn write(self, world: &mut World) {
+        self(world);
+    }
+}
+
 #[derive(Debug)]
 pub struct Spawn<T> {
     pub bundle: T,
@@ -713,23 +749,33 @@ where
     }
 }
 
-pub struct InsertResource<T: Resource> {
-    pub resource: T,
+pub struct InitResource<R: Resource + FromWorld> {
+    _phantom: PhantomData<R>,
 }
 
-impl<T: Resource> Command for InsertResource<T> {
+impl<R: Resource + FromWorld> Command for InitResource<R> {
+    fn write(self, world: &mut World) {
+        world.init_resource::<R>();
+    }
+}
+
+pub struct InsertResource<R: Resource> {
+    pub resource: R,
+}
+
+impl<R: Resource> Command for InsertResource<R> {
     fn write(self, world: &mut World) {
         world.insert_resource(self.resource);
     }
 }
 
-pub struct RemoveResource<T: Resource> {
-    pub phantom: PhantomData<T>,
+pub struct RemoveResource<R: Resource> {
+    pub phantom: PhantomData<R>,
 }
 
-impl<T: Resource> Command for RemoveResource<T> {
+impl<R: Resource> Command for RemoveResource<R> {
     fn write(self, world: &mut World) {
-        world.remove_resource::<T>();
+        world.remove_resource::<R>();
     }
 }
 
@@ -769,6 +815,10 @@ mod tests {
     #[derive(Component)]
     struct W<T>(T);
 
+    fn simple_command(world: &mut World) {
+        world.spawn().insert_bundle((W(0u32), W(42u64)));
+    }
+
     #[test]
     fn commands() {
         let mut world = World::default();
@@ -797,6 +847,27 @@ mod tests {
             .map(|(a, b)| (a.0, b.0))
             .collect::<Vec<_>>();
         assert_eq!(results2, vec![]);
+
+        // test adding simple (FnOnce) commands
+        {
+            let mut commands = Commands::new(&mut command_queue, &world);
+
+            // set up a simple command using a closure that adds one additional entity
+            commands.add(|world: &mut World| {
+                world.spawn().insert_bundle((W(42u32), W(0u64)));
+            });
+
+            // set up a simple command using a function that adds one additional entity
+            commands.add(simple_command);
+        }
+        command_queue.apply(&mut world);
+        let results3 = world
+            .query::<(&W<u32>, &W<u64>)>()
+            .iter(&world)
+            .map(|(a, b)| (a.0, b.0))
+            .collect::<Vec<_>>();
+
+        assert_eq!(results3, vec![(42u32, 0u64), (0u32, 42u64)]);
     }
 
     #[test]
