@@ -6,13 +6,14 @@ use crate::{
     component::{Component, ComponentId, ComponentTicks, Components},
     entity::{Entities, Entity},
     query::{
-        FilterFetch, FilteredAccess, FilteredAccessSet, QueryState, ReadOnlyFetch, WorldQuery,
+        Access, FilterFetch, FilteredAccess, FilteredAccessSet, QueryState, ReadOnlyFetch,
+        WorldQuery,
     },
     system::{CommandQueue, Commands, Query, SystemMeta},
     world::{FromWorld, World},
 };
 pub use bevy_ecs_macros::SystemParam;
-use bevy_ecs_macros::{all_tuples, impl_query_set};
+use bevy_ecs_macros::{all_tuples, impl_param_set};
 use std::{
     fmt::Debug,
     marker::PhantomData,
@@ -42,7 +43,7 @@ use std::{
 ///     // Access the resource through `param.foo`
 /// }
 ///
-/// # my_system.system();
+/// # bevy_ecs::system::assert_is_system(my_system);
 /// ```
 pub trait SystemParam: Sized {
     type Fetch: for<'w, 's> SystemParamFetch<'w, 's>;
@@ -59,25 +60,11 @@ pub type SystemParamItem<'w, 's, P> = <<P as SystemParam>::Fetch as SystemParamF
 /// Additionally, it is the implementor's responsibility to ensure there is no
 /// conflicting access across all [`SystemParam`]'s.
 pub unsafe trait SystemParamState: Send + Sync + 'static {
-    /// Values of this type can be used to adjust the behavior of the
-    /// system parameter. For instance, this can be used to pass
-    /// values from a `Plugin` to a `System`, or to control the
-    /// behavior of the `System`.
-    ///
-    /// The default configuration of the parameter is set by
-    /// [`SystemParamState::default_config`]. To change it, invoke
-    /// [`FunctionSystem::config`](super::FunctionSystem::config) when
-    /// creating the system.
-    ///
-    /// See [`FunctionSystem::config`](super::FunctionSystem::config)
-    /// for more information and examples.
-    type Config: Send + Sync;
-    fn init(world: &mut World, system_meta: &mut SystemMeta, config: Self::Config) -> Self;
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self;
     #[inline]
     fn new_archetype(&mut self, _archetype: &Archetype, _system_meta: &mut SystemMeta) {}
     #[inline]
     fn apply(&mut self, _world: &mut World) {}
-    fn default_config() -> Self::Config;
 }
 
 /// A [`SystemParamFetch`] that only reads a given [`World`].
@@ -87,7 +74,7 @@ pub unsafe trait SystemParamState: Send + Sync + 'static {
 pub unsafe trait ReadOnlySystemParamFetch {}
 
 pub trait SystemParamFetch<'world, 'state>: SystemParamState {
-    type Item;
+    type Item: SystemParam<Fetch = Self>;
     /// # Safety
     ///
     /// This call might access any of the input parameters in an unsafe way. Make sure the data
@@ -121,9 +108,7 @@ unsafe impl<Q: WorldQuery + 'static, F: WorldQuery + 'static> SystemParamState f
 where
     F::Fetch: FilterFetch,
 {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
         let state = QueryState::new(world);
         assert_component_access_compatibility(
             &system_meta.name,
@@ -148,8 +133,6 @@ where
             .archetype_component_access
             .extend(&self.archetype_component_access);
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, Q: WorldQuery + 'static, F: WorldQuery + 'static> SystemParamFetch<'w, 's>
@@ -187,22 +170,23 @@ fn assert_component_access_compatibility(
         .map(|component_id| world.components.get_info(component_id).unwrap().name())
         .collect::<Vec<&str>>();
     let accesses = conflicting_components.join(", ");
-    panic!("error[B0001]: Query<{}, {}> in system {} accesses component(s) {} in a way that conflicts with a previous system parameter. Consider using `Without<T>` to create disjoint Queries or merging conflicting Queries into a `QuerySet`.",
-                query_type, filter_type, system_name, accesses);
+    panic!("error[B0001]: Query<{}, {}> in system {} accesses component(s) {} in a way that conflicts with a previous system parameter. Consider using `Without<T>` to create disjoint Queries or merging conflicting Queries into a `ParamSet`.",
+           query_type, filter_type, system_name, accesses);
 }
 
-pub struct QuerySet<'w, 's, T> {
-    query_states: &'s T,
+pub struct ParamSet<'w, 's, T: SystemParam> {
+    param_states: &'s mut T::Fetch,
     world: &'w World,
-    last_change_tick: u32,
+    system_meta: SystemMeta,
     change_tick: u32,
 }
+/// The [`SystemParamState`] of [`ParamSet<T::Item>`].
+pub struct ParamSetState<T: for<'w, 's> SystemParamFetch<'w, 's>>(T);
 
-pub struct QuerySetState<T>(T);
-
-impl_query_set!();
+impl_param_set!();
 
 pub trait Resource: Send + Sync + 'static {}
+
 impl<T> Resource for T where T: Send + Sync + 'static {}
 
 /// Shared borrow of a resource.
@@ -269,7 +253,19 @@ impl<'w, T: Resource> AsRef<T> for Res<'w, T> {
     }
 }
 
+impl<'w, T: Resource> From<ResMut<'w, T>> for Res<'w, T> {
+    fn from(res: ResMut<'w, T>) -> Self {
+        Self {
+            value: res.value,
+            ticks: res.ticks.component_ticks,
+            change_tick: res.ticks.change_tick,
+            last_change_tick: res.ticks.last_change_tick,
+        }
+    }
+}
+
 /// The [`SystemParamState`] of [`Res<T>`].
+#[doc(hidden)]
 pub struct ResState<T> {
     component_id: ComponentId,
     marker: PhantomData<T>,
@@ -282,16 +278,15 @@ impl<'a, T: Resource> SystemParam for Res<'a, T> {
 // SAFE: Res ComponentId and ArchetypeComponentId access is applied to SystemMeta. If this Res
 // conflicts with any prior access, a panic will occur.
 unsafe impl<T: Resource> SystemParamState for ResState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
         let component_id = world.initialize_resource::<T>();
         let combined_access = system_meta.component_access_set.combined_access_mut();
-        if combined_access.has_write(component_id) {
-            panic!(
-                "error[B0002]: Res<{}> in system {} conflicts with a previous ResMut<{0}> access. Consider removing the duplicate access.",
-                std::any::type_name::<T>(), system_meta.name);
-        }
+        assert!(
+            !combined_access.has_write(component_id),
+            "error[B0002]: Res<{}> in system {} conflicts with a previous ResMut<{0}> access. Consider removing the duplicate access.",
+            std::any::type_name::<T>(),
+            system_meta.name,
+        );
         combined_access.add_read(component_id);
 
         let resource_archetype = world.archetypes.resource();
@@ -306,8 +301,6 @@ unsafe impl<T: Resource> SystemParamState for ResState<T> {
             marker: PhantomData,
         }
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: Resource> SystemParamFetch<'w, 's> for ResState<T> {
@@ -340,6 +333,7 @@ impl<'w, 's, T: Resource> SystemParamFetch<'w, 's> for ResState<T> {
 
 /// The [`SystemParamState`] of [`Option<Res<T>>`].
 /// See: [`Res<T>`]
+#[doc(hidden)]
 pub struct OptionResState<T>(ResState<T>);
 
 impl<'a, T: Resource> SystemParam for Option<Res<'a, T>> {
@@ -350,13 +344,9 @@ impl<'a, T: Resource> SystemParam for Option<Res<'a, T>> {
 unsafe impl<T: Resource> ReadOnlySystemParamFetch for OptionResState<T> {}
 
 unsafe impl<T: Resource> SystemParamState for OptionResState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
-        Self(ResState::init(world, system_meta, ()))
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
+        Self(ResState::init(world, system_meta))
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: Resource> SystemParamFetch<'w, 's> for OptionResState<T> {
@@ -381,6 +371,7 @@ impl<'w, 's, T: Resource> SystemParamFetch<'w, 's> for OptionResState<T> {
 }
 
 /// The [`SystemParamState`] of [`ResMut<T>`].
+#[doc(hidden)]
 pub struct ResMutState<T> {
     component_id: ComponentId,
     marker: PhantomData<T>,
@@ -393,9 +384,7 @@ impl<'a, T: Resource> SystemParam for ResMut<'a, T> {
 // SAFE: Res ComponentId and ArchetypeComponentId access is applied to SystemMeta. If this Res
 // conflicts with any prior access, a panic will occur.
 unsafe impl<T: Resource> SystemParamState for ResMutState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
         let component_id = world.initialize_resource::<T>();
         let combined_access = system_meta.component_access_set.combined_access_mut();
         if combined_access.has_write(component_id) {
@@ -421,8 +410,6 @@ unsafe impl<T: Resource> SystemParamState for ResMutState<T> {
             marker: PhantomData,
         }
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: Resource> SystemParamFetch<'w, 's> for ResMutState<T> {
@@ -457,6 +444,7 @@ impl<'w, 's, T: Resource> SystemParamFetch<'w, 's> for ResMutState<T> {
 
 /// The [`SystemParamState`] of [`Option<ResMut<T>>`].
 /// See: [`ResMut<T>`]
+#[doc(hidden)]
 pub struct OptionResMutState<T>(ResMutState<T>);
 
 impl<'a, T: Resource> SystemParam for Option<ResMut<'a, T>> {
@@ -464,13 +452,9 @@ impl<'a, T: Resource> SystemParam for Option<ResMut<'a, T>> {
 }
 
 unsafe impl<T: Resource> SystemParamState for OptionResMutState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
-        Self(ResMutState::init(world, system_meta, ()))
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
+        Self(ResMutState::init(world, system_meta))
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: Resource> SystemParamFetch<'w, 's> for OptionResMutState<T> {
@@ -505,17 +489,13 @@ unsafe impl ReadOnlySystemParamFetch for CommandQueue {}
 
 // SAFE: only local state is accessed
 unsafe impl SystemParamState for CommandQueue {
-    type Config = ();
-
-    fn init(_world: &mut World, _system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(_world: &mut World, _system_meta: &mut SystemMeta) -> Self {
         Default::default()
     }
 
     fn apply(&mut self, world: &mut World) {
         self.apply(world);
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's> SystemParamFetch<'w, 's> for CommandQueue {
@@ -532,6 +512,57 @@ impl<'w, 's> SystemParamFetch<'w, 's> for CommandQueue {
     }
 }
 
+/// SAFE: only reads world
+unsafe impl ReadOnlySystemParamFetch for WorldState {}
+
+/// The [`SystemParamState`] of [`&World`](crate::world::World).
+#[doc(hidden)]
+pub struct WorldState;
+
+impl<'w, 's> SystemParam for &'w World {
+    type Fetch = WorldState;
+}
+
+unsafe impl<'w, 's> SystemParamState for WorldState {
+    fn init(_world: &mut World, system_meta: &mut SystemMeta) -> Self {
+        let mut access = Access::default();
+        access.read_all();
+        if !system_meta
+            .archetype_component_access
+            .is_compatible(&access)
+        {
+            panic!("&World conflicts with a previous mutable system parameter. Allowing this would break Rust's mutability rules");
+        }
+        system_meta.archetype_component_access.extend(&access);
+
+        let mut filtered_access = FilteredAccess::default();
+
+        filtered_access.read_all();
+        if !system_meta
+            .component_access_set
+            .get_conflicts(&filtered_access)
+            .is_empty()
+        {
+            panic!("&World conflicts with a previous mutable system parameter. Allowing this would break Rust's mutability rules");
+        }
+        system_meta.component_access_set.add(filtered_access);
+
+        WorldState
+    }
+}
+
+impl<'w, 's> SystemParamFetch<'w, 's> for WorldState {
+    type Item = &'w World;
+    unsafe fn get_param(
+        _state: &'s mut Self,
+        _system_meta: &SystemMeta,
+        world: &'w World,
+        _change_tick: u32,
+    ) -> Self::Item {
+        world
+    }
+}
+
 /// A system local [`SystemParam`].
 ///
 /// A local may only be accessed by the system itself and is therefore not visible to other systems.
@@ -548,8 +579,8 @@ impl<'w, 's> SystemParamFetch<'w, 's> for CommandQueue {
 /// fn read_from_local(local: Local<usize>) -> usize {
 ///     *local
 /// }
-/// let mut write_system = write_to_local.system();
-/// let mut read_system = read_from_local.system();
+/// let mut write_system = IntoSystem::into_system(write_to_local);
+/// let mut read_system = IntoSystem::into_system(read_from_local);
 /// write_system.initialize(world);
 /// read_system.initialize(world);
 ///
@@ -558,6 +589,22 @@ impl<'w, 's> SystemParamFetch<'w, 's> for CommandQueue {
 /// // Note how the read local is still 0 due to the locals not being shared.
 /// assert_eq!(read_system.run((), world), 0);
 /// ```
+///
+/// N.B. A [`Local`]s value cannot be read or written to outside of the containing system.
+/// To add configuration to a system, convert a capturing closure into the system instead:
+///
+/// ```
+/// # use bevy_ecs::prelude::*;
+/// # use bevy_ecs::system::assert_is_system;
+/// struct Config(u32);
+/// struct Myu32Wrapper(u32);
+/// fn reset_to_system(value: Config) -> impl FnMut(ResMut<Myu32Wrapper>) {
+///     move |mut val| val.0 = value.0
+/// }
+///
+/// // .add_system(reset_to_system(my_config))
+/// # assert_is_system(reset_to_system(Config(10)));
+/// ```
 pub struct Local<'a, T: Resource>(&'a mut T);
 
 // SAFE: Local only accesses internal state
@@ -589,6 +636,7 @@ impl<'a, T: Resource> DerefMut for Local<'a, T> {
 }
 
 /// The [`SystemParamState`] of [`Local<T>`].
+#[doc(hidden)]
 pub struct LocalState<T: Resource>(T);
 
 impl<'a, T: Resource + FromWorld> SystemParam for Local<'a, T> {
@@ -597,14 +645,8 @@ impl<'a, T: Resource + FromWorld> SystemParam for Local<'a, T> {
 
 // SAFE: only local state is accessed
 unsafe impl<T: Resource + FromWorld> SystemParamState for LocalState<T> {
-    type Config = Option<T>;
-
-    fn init(world: &mut World, _system_meta: &mut SystemMeta, config: Self::Config) -> Self {
-        Self(config.unwrap_or_else(|| T::from_world(world)))
-    }
-
-    fn default_config() -> Option<T> {
-        None
+    fn init(world: &mut World, _system_meta: &mut SystemMeta) -> Self {
+        Self(T::from_world(world))
     }
 }
 
@@ -634,7 +676,9 @@ impl<'w, 's, T: Resource + FromWorld> SystemParamFetch<'w, 's> for LocalState<T>
 /// and will need to be manually flushed using [`World::clear_trackers`]
 ///
 /// For users of `bevy` itself, this is automatically done in a system added by `MinimalPlugins`
-/// or `DefaultPlugins` at the end of each pass of the game loop.
+/// or `DefaultPlugins` at the end of each pass of the game loop during the `CoreStage::Last`
+/// stage. As such `RemovedComponents` systems should be scheduled after the stage where
+/// removal occurs but before `CoreStage::Last`.
 ///
 /// # Examples
 ///
@@ -652,7 +696,7 @@ impl<'w, 's, T: Resource + FromWorld> SystemParamFetch<'w, 's> for LocalState<T>
 ///     removed.iter().for_each(|removed_entity| println!("{:?}", removed_entity));
 /// }
 ///
-/// # react_on_removal.system();
+/// # bevy_ecs::system::assert_is_system(react_on_removal);
 /// ```
 pub struct RemovedComponents<'a, T: Component> {
     world: &'a World,
@@ -671,6 +715,7 @@ impl<'a, T: Component> RemovedComponents<'a, T> {
 unsafe impl<T: Component> ReadOnlySystemParamFetch for RemovedComponentsState<T> {}
 
 /// The [`SystemParamState`] of [`RemovedComponents<T>`].
+#[doc(hidden)]
 pub struct RemovedComponentsState<T> {
     component_id: ComponentId,
     marker: PhantomData<T>,
@@ -683,16 +728,12 @@ impl<'a, T: Component> SystemParam for RemovedComponents<'a, T> {
 // SAFE: no component access. removed component entity collections can be read in parallel and are
 // never mutably borrowed during system execution
 unsafe impl<T: Component> SystemParamState for RemovedComponentsState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, _system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(world: &mut World, _system_meta: &mut SystemMeta) -> Self {
         Self {
             component_id: world.init_component::<T>(),
             marker: PhantomData,
         }
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: Component> SystemParamFetch<'w, 's> for RemovedComponentsState<T> {
@@ -766,8 +807,19 @@ impl<'w, T> Deref for NonSend<'w, T> {
         self.value
     }
 }
+impl<'a, T> From<NonSendMut<'a, T>> for NonSend<'a, T> {
+    fn from(nsm: NonSendMut<'a, T>) -> Self {
+        Self {
+            value: nsm.value,
+            ticks: nsm.ticks.component_ticks.to_owned(),
+            change_tick: nsm.ticks.change_tick,
+            last_change_tick: nsm.ticks.last_change_tick,
+        }
+    }
+}
 
 /// The [`SystemParamState`] of [`NonSend<T>`].
+#[doc(hidden)]
 pub struct NonSendState<T> {
     component_id: ComponentId,
     marker: PhantomData<fn() -> T>,
@@ -780,18 +832,17 @@ impl<'a, T: 'static> SystemParam for NonSend<'a, T> {
 // SAFE: NonSendComponentId and ArchetypeComponentId access is applied to SystemMeta. If this
 // NonSend conflicts with any prior access, a panic will occur.
 unsafe impl<T: 'static> SystemParamState for NonSendState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
         system_meta.set_non_send();
 
         let component_id = world.initialize_non_send_resource::<T>();
         let combined_access = system_meta.component_access_set.combined_access_mut();
-        if combined_access.has_write(component_id) {
-            panic!(
-                "error[B0002]: NonSend<{}> in system {} conflicts with a previous mutable resource access ({0}). Consider removing the duplicate access.",
-                std::any::type_name::<T>(), system_meta.name);
-        }
+        assert!(
+            !combined_access.has_write(component_id),
+            "error[B0002]: NonSend<{}> in system {} conflicts with a previous mutable resource access ({0}). Consider removing the duplicate access.",
+            std::any::type_name::<T>(),
+            system_meta.name,
+        );
         combined_access.add_read(component_id);
 
         let resource_archetype = world.archetypes.resource();
@@ -806,8 +857,6 @@ unsafe impl<T: 'static> SystemParamState for NonSendState<T> {
             marker: PhantomData,
         }
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: 'static> SystemParamFetch<'w, 's> for NonSendState<T> {
@@ -842,6 +891,7 @@ impl<'w, 's, T: 'static> SystemParamFetch<'w, 's> for NonSendState<T> {
 
 /// The [`SystemParamState`] of [`Option<NonSend<T>>`].
 /// See: [`NonSend<T>`]
+#[doc(hidden)]
 pub struct OptionNonSendState<T>(NonSendState<T>);
 
 impl<'w, T: 'static> SystemParam for Option<NonSend<'w, T>> {
@@ -852,13 +902,9 @@ impl<'w, T: 'static> SystemParam for Option<NonSend<'w, T>> {
 unsafe impl<T: 'static> ReadOnlySystemParamFetch for OptionNonSendState<T> {}
 
 unsafe impl<T: 'static> SystemParamState for OptionNonSendState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
-        Self(NonSendState::init(world, system_meta, ()))
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
+        Self(NonSendState::init(world, system_meta))
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: 'static> SystemParamFetch<'w, 's> for OptionNonSendState<T> {
@@ -884,6 +930,7 @@ impl<'w, 's, T: 'static> SystemParamFetch<'w, 's> for OptionNonSendState<T> {
 }
 
 /// The [`SystemParamState`] of [`NonSendMut<T>`].
+#[doc(hidden)]
 pub struct NonSendMutState<T> {
     component_id: ComponentId,
     marker: PhantomData<fn() -> T>,
@@ -896,9 +943,7 @@ impl<'a, T: 'static> SystemParam for NonSendMut<'a, T> {
 // SAFE: NonSendMut ComponentId and ArchetypeComponentId access is applied to SystemMeta. If this
 // NonSendMut conflicts with any prior access, a panic will occur.
 unsafe impl<T: 'static> SystemParamState for NonSendMutState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
         system_meta.set_non_send();
 
         let component_id = world.initialize_non_send_resource::<T>();
@@ -926,8 +971,6 @@ unsafe impl<T: 'static> SystemParamState for NonSendMutState<T> {
             marker: PhantomData,
         }
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: 'static> SystemParamFetch<'w, 's> for NonSendMutState<T> {
@@ -963,6 +1006,7 @@ impl<'w, 's, T: 'static> SystemParamFetch<'w, 's> for NonSendMutState<T> {
 
 /// The [`SystemParamState`] of [`Option<NonSendMut<T>>`].
 /// See: [`NonSendMut<T>`]
+#[doc(hidden)]
 pub struct OptionNonSendMutState<T>(NonSendMutState<T>);
 
 impl<'a, T: 'static> SystemParam for Option<NonSendMut<'a, T>> {
@@ -970,13 +1014,9 @@ impl<'a, T: 'static> SystemParam for Option<NonSendMut<'a, T>> {
 }
 
 unsafe impl<T: 'static> SystemParamState for OptionNonSendMutState<T> {
-    type Config = ();
-
-    fn init(world: &mut World, system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
-        Self(NonSendMutState::init(world, system_meta, ()))
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
+        Self(NonSendMutState::init(world, system_meta))
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's, T: 'static> SystemParamFetch<'w, 's> for OptionNonSendMutState<T> {
@@ -1011,17 +1051,14 @@ impl<'a> SystemParam for &'a Archetypes {
 unsafe impl ReadOnlySystemParamFetch for ArchetypesState {}
 
 /// The [`SystemParamState`] of [`Archetypes`].
+#[doc(hidden)]
 pub struct ArchetypesState;
 
 // SAFE: no component value access
 unsafe impl SystemParamState for ArchetypesState {
-    type Config = ();
-
-    fn init(_world: &mut World, _system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(_world: &mut World, _system_meta: &mut SystemMeta) -> Self {
         Self
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's> SystemParamFetch<'w, 's> for ArchetypesState {
@@ -1046,17 +1083,14 @@ impl<'a> SystemParam for &'a Components {
 unsafe impl ReadOnlySystemParamFetch for ComponentsState {}
 
 /// The [`SystemParamState`] of [`Components`].
+#[doc(hidden)]
 pub struct ComponentsState;
 
 // SAFE: no component value access
 unsafe impl SystemParamState for ComponentsState {
-    type Config = ();
-
-    fn init(_world: &mut World, _system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(_world: &mut World, _system_meta: &mut SystemMeta) -> Self {
         Self
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's> SystemParamFetch<'w, 's> for ComponentsState {
@@ -1081,17 +1115,14 @@ impl<'a> SystemParam for &'a Entities {
 unsafe impl ReadOnlySystemParamFetch for EntitiesState {}
 
 /// The [`SystemParamState`] of [`Entities`].
+#[doc(hidden)]
 pub struct EntitiesState;
 
 // SAFE: no component value access
 unsafe impl SystemParamState for EntitiesState {
-    type Config = ();
-
-    fn init(_world: &mut World, _system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(_world: &mut World, _system_meta: &mut SystemMeta) -> Self {
         Self
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's> SystemParamFetch<'w, 's> for EntitiesState {
@@ -1116,17 +1147,14 @@ impl<'a> SystemParam for &'a Bundles {
 unsafe impl ReadOnlySystemParamFetch for BundlesState {}
 
 /// The [`SystemParamState`] of [`Bundles`].
+#[doc(hidden)]
 pub struct BundlesState;
 
 // SAFE: no component value access
 unsafe impl SystemParamState for BundlesState {
-    type Config = ();
-
-    fn init(_world: &mut World, _system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(_world: &mut World, _system_meta: &mut SystemMeta) -> Self {
         Self
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's> SystemParamFetch<'w, 's> for BundlesState {
@@ -1143,6 +1171,7 @@ impl<'w, 's> SystemParamFetch<'w, 's> for BundlesState {
     }
 }
 
+/// The [`SystemParamState`] of [`SystemChangeTick`].
 #[derive(Debug)]
 pub struct SystemChangeTick {
     pub last_change_tick: u32,
@@ -1157,16 +1186,13 @@ impl SystemParam for SystemChangeTick {
 }
 
 /// The [`SystemParamState`] of [`SystemChangeTick`].
+#[doc(hidden)]
 pub struct SystemChangeTickState {}
 
 unsafe impl SystemParamState for SystemChangeTickState {
-    type Config = ();
-
-    fn init(_world: &mut World, _system_meta: &mut SystemMeta, _config: Self::Config) -> Self {
+    fn init(_world: &mut World, _system_meta: &mut SystemMeta) -> Self {
         Self {}
     }
-
-    fn default_config() {}
 }
 
 impl<'w, 's> SystemParamFetch<'w, 's> for SystemChangeTickState {
@@ -1216,11 +1242,9 @@ macro_rules! impl_system_param_tuple {
         /// SAFE: implementors of each `SystemParamState` in the tuple have validated their impls
         #[allow(non_snake_case)]
         unsafe impl<$($param: SystemParamState),*> SystemParamState for ($($param,)*) {
-            type Config = ($(<$param as SystemParamState>::Config,)*);
             #[inline]
-            fn init(_world: &mut World, _system_meta: &mut SystemMeta, config: Self::Config) -> Self {
-                let ($($param,)*) = config;
-                (($($param::init(_world, _system_meta, $param),)*))
+            fn init(_world: &mut World, _system_meta: &mut SystemMeta) -> Self {
+                (($($param::init(_world, _system_meta),)*))
             }
 
             #[inline]
@@ -1234,11 +1258,6 @@ macro_rules! impl_system_param_tuple {
                 let ($($param,)*) = self;
                 $($param.apply(_world);)*
             }
-
-            #[allow(clippy::unused_unit)]
-            fn default_config() -> ($(<$param as SystemParamState>::Config,)*) {
-                ($(<$param as SystemParamState>::default_config(),)*)
-            }
         }
     };
 }
@@ -1253,3 +1272,153 @@ pub mod lifetimeless {
     pub type SResMut<T> = super::ResMut<'static, T>;
     pub type SCommands = crate::system::Commands<'static, 'static>;
 }
+
+/// A helper for using system parameters in generic contexts
+///
+/// This type is a [`SystemParam`] adapter which always has
+/// `Self::Fetch::Item == Self` (ignoring lifetimes for brevity),
+/// no matter the argument [`SystemParam`] (`P`) (other than
+/// that `P` must be `'static`)
+///
+/// This makes it useful for having arbitrary [`SystemParam`] type arguments
+/// to function systems, or for generic types using the [`derive@SystemParam`]
+/// derive:
+///
+/// ```
+/// # use bevy_ecs::prelude::*;
+/// use bevy_ecs::system::{SystemParam, StaticSystemParam};
+/// #[derive(SystemParam)]
+/// struct GenericParam<'w,'s, T: SystemParam + 'static> {
+///     field: StaticSystemParam<'w, 's, T>,
+/// }
+/// fn do_thing_generically<T: SystemParam + 'static>(t: StaticSystemParam<T>) {}
+///
+/// fn check_always_is_system<T: SystemParam + 'static>(){
+///     bevy_ecs::system::assert_is_system(do_thing_generically::<T>);
+/// }
+/// ```
+/// Note that in a real case you'd generally want
+/// additional bounds on `P`, for your use of the parameter
+/// to have a reason to be generic.
+///
+/// For example, using this would allow a type to be generic over
+/// whether a resource is accessed mutably or not, with
+/// impls being bounded on [`P: Deref<Target=MyType>`](Deref), and
+/// [`P: DerefMut<Target=MyType>`](DerefMut) depending on whether the
+/// method requires mutable access or not.
+///
+/// The method which doesn't use this type will not compile:
+/// ```compile_fail
+/// # use bevy_ecs::prelude::*;
+/// # use bevy_ecs::system::{SystemParam, StaticSystemParam};
+///
+/// fn do_thing_generically<T: SystemParam + 'static>(t: T) {}
+///
+/// #[derive(SystemParam)]
+/// struct GenericParam<'w,'s, T: SystemParam> {
+///     field: T,
+///     #[system_param(ignore)]
+///     // Use the lifetimes, as the `SystemParam` derive requires them
+///     phantom: core::marker::PhantomData<&'w &'s ()>
+/// }
+/// # fn check_always_is_system<T: SystemParam + 'static>(){
+/// #    bevy_ecs::system::assert_is_system(do_thing_generically::<T>);
+/// # }
+/// ```
+///
+pub struct StaticSystemParam<'w, 's, P: SystemParam>(SystemParamItem<'w, 's, P>);
+
+impl<'w, 's, P: SystemParam> Deref for StaticSystemParam<'w, 's, P> {
+    type Target = SystemParamItem<'w, 's, P>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl<'w, 's, P: SystemParam> DerefMut for StaticSystemParam<'w, 's, P> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.0
+    }
+}
+
+impl<'w, 's, P: SystemParam> StaticSystemParam<'w, 's, P> {
+    /// Get the value of the parameter
+    pub fn into_inner(self) -> SystemParamItem<'w, 's, P> {
+        self.0
+    }
+}
+
+/// The [`SystemParamState`] of [`StaticSystemParam`].
+#[doc(hidden)]
+pub struct StaticSystemParamState<S, P>(S, PhantomData<fn() -> P>);
+
+// Safe: This doesn't add any more reads, and the delegated fetch confirms it
+unsafe impl<'w, 's, S: ReadOnlySystemParamFetch, P> ReadOnlySystemParamFetch
+    for StaticSystemParamState<S, P>
+{
+}
+
+impl<'world, 'state, P: SystemParam + 'static> SystemParam
+    for StaticSystemParam<'world, 'state, P>
+{
+    type Fetch = StaticSystemParamState<P::Fetch, P>;
+}
+
+impl<'world, 'state, S: SystemParamFetch<'world, 'state>, P: SystemParam + 'static>
+    SystemParamFetch<'world, 'state> for StaticSystemParamState<S, P>
+where
+    P: SystemParam<Fetch = S>,
+{
+    type Item = StaticSystemParam<'world, 'state, P>;
+
+    unsafe fn get_param(
+        state: &'state mut Self,
+        system_meta: &SystemMeta,
+        world: &'world World,
+        change_tick: u32,
+    ) -> Self::Item {
+        // Safe: We properly delegate SystemParamState
+        StaticSystemParam(S::get_param(&mut state.0, system_meta, world, change_tick))
+    }
+}
+
+unsafe impl<'w, 's, S: SystemParamState, P: SystemParam + 'static> SystemParamState
+    for StaticSystemParamState<S, P>
+{
+    fn init(world: &mut World, system_meta: &mut SystemMeta) -> Self {
+        Self(S::init(world, system_meta), PhantomData)
+    }
+
+    fn new_archetype(&mut self, archetype: &Archetype, system_meta: &mut SystemMeta) {
+        self.0.new_archetype(archetype, system_meta)
+    }
+
+    fn apply(&mut self, world: &mut World) {
+        self.0.apply(world)
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::SystemParam;
+    use crate::{
+        self as bevy_ecs, // Necessary for the `SystemParam` Derive when used inside `bevy_ecs`.
+        query::{FilterFetch, WorldQuery},
+        system::Query,
+    };
+
+    // Compile test for #2838
+    #[derive(SystemParam)]
+    pub struct SpecialQuery<
+        'w,
+        's,
+        Q: WorldQuery + Send + Sync + 'static,
+        F: WorldQuery + Send + Sync + 'static = (),
+    >
+    where
+        F::Fetch: FilterFetch,
+    {
+        _query: Query<'w, 's, Q, F>,
+    }
+}
